/*
 *  AudioFeature Framework
 *	Compute some low-level audio descriptors.
 *  
 *  Marcelo Cicconet, Visgraf / IMPA
 *	www.impa.br/~cicconet
 *
 *	This framework is distributed under the MIT License.
 *	See notice at the end of this file.
 *
 */

#import <vecLib/vDSP.h>

@interface AFAudioDescriptors : NSObject {
	UInt32 windowSize;
	UInt32 halfWindowSize;
	
	// for temporary use in some methods
	UInt32 windowSizeInBytes;
	UInt32 halfWindowSizeInBytes;
	float * bufferOfWindowSize; // size: windowSize
	float * bufferOfHalfWindowSize; // size: halfWindowSize
	float sumFFTMag;
	
	// fft setup
	vDSP_Length length;
	FFTRadix fftRadix;
	FFTSetup fftSetup;
	FFTDirection fftDirectionFw;
	DSPSplitComplex dspSplitComplexIO;
	float * hannWindow;
	
	// pre-chroma vector setup
	float frequencyLag;
	UInt8 halfNumIndBellowWindow;
	UInt8 numIndBellowWindow;
	float hannWindowSize;
	
	// chroma vector setup
	float minFrequency;
	float maxFrequency;
	
	// bark scale setup
	UInt32 * barkBandEdges;
}

// ----------------------------------------------------------------------------------------------------
#pragma mark Initialization
// ----------------------------------------------------------------------------------------------------
- (id)initWithForwardFFTSetupForWindowSize:(UInt32)theWindowSize;

// ----------------------------------------------------------------------------------------------------
#pragma mark 1-Dimensional Audio Features
// ----------------------------------------------------------------------------------------------------
- (float)loudnessOfAudioWindow:(float *)theAudioWindow;
// sum of the squared magnitudes of the spectrum of frequencies

- (float)spectralCentroidOfAudioWindow:(float *)theAudioWindow;
// barycenter of the magnitudes of the spectrum of frequencies

- (float)spectralSpreadOfAudioWindow:(float *)theAudioWindow;
// variance of the magnitudes of the spectrum of frequencies

- (float)spectralSkewnessOfAudioWindow:(float *)theAudioWindow;
// 3rd order moment of the magnitudes of the spectrum of frequencies
// skewness = 0: symmetric distribution
// skewness < 0: more energy on the right
// skewness > 0: more energy on the left

- (float)spectralRollOffOfAudioWindow:(float *)theAudioWindow;
// frequency so that 95% of the signal energy is contained bellow it
// it is correlated somehow to the noise cutting frequency

- (float)spectralFlatnessOfAudioWindow:(float *)theAudioWindow;
// ratio of the geometric mean to the arithmetic mean of the magnitudes of the spectrum of frequencies
// its a measure of the noisiness (flatness) of the spectrum of the spectrum magnitudes

- (float)spectralCrestOfAudioWindow:(float *)theAudioWindow;
// ratio of the maximum value to the arithmetic mean of the magnitudes of the spectrum of frequencies

// ----------------------------------------------------------------------------------------------------
#pragma mark N-Dimensional Audio Features
// ----------------------------------------------------------------------------------------------------
- (void)preChroma:(float *)thePreChroma ofAudioWindow:(float *)theAudioWindow; // 84-dimensional
// 84 piano frequency energies (from midi note 24 to midi note 107)
// entries are in the range [0,1]

- (void)chroma:(float *)theChroma ofAudioWindow:(float *)theAudioWindow; // 12-dimensional
// frequency energies for each of the 12 keys (from C to B)
// entries are in the range [0,1]

- (void)barkBands:(float *)theBarkBands ofAudioWindow:(float *)theAudioWindow; // 25-dimensional
// sum of the squared magnitudes of the spectrum in 25 Bark bands

// ----------------------------------------------------------------------------------------------------
#pragma mark Auxiliary Routines
// ----------------------------------------------------------------------------------------------------
- (void)inPlaceForwardFFTOfComplexVector:(DSPSplitComplex)theComplexVector;
- (void)fftMagnitudes:(float *)theMagnitudes ofComplexVector:(DSPSplitComplex)theComplexVector;
- (void)fftMagnitudes:(float *)theMagnitudes ofRealVector:(float *)theRealVector;
- (void)powerSpectrum:(float *)thePowerSpectrum ofComplexVector:(DSPSplitComplex)theComplexVector;
- (void)powerSpectrum:(float *)thePowerSpectrum ofRealVector:(float *)theRealVector;
- (void)preChromaVector:(float *)thePreChromaVector fromPowerSpectrum:(float *)thePowerSpectrum;
- (void)chromaVector:(float *)theChromaVector fromPowerSpectrum:(float *)thePowerSpectrum;
- (void)barkScaleWeights:(float *)theBarkScaleWeights fromPowerSpectrum:(float *)thePowerSpectrum;
- (float)loudnessFromPowerSpectrum:(float *)thePowerSpectrum;
- (float)highFrequencyContentFromPowerSpectrum:(float *)thePowerSpectrum;
- (float)chromaWeightForDistance:(float)theDistance;
- (void)clearImagPartOfSplitComplex;
- (void)inPlaceWindowingOfData:(float *)theData;
- (float)evaluateHannWindowWithCenter:(float)theCenter andSize:(float)theSize atPoint:(float)thePoint;
- (float)hertzToMel:(float)theHertzFrequency;

- (void)dealloc;

@end

/*
 *	Copyright (C) 2011 by Marcelo Cicconet
 *
 *	Permission is hereby granted, free of charge, to any person obtaining a copy
 *	of this software and associated documentation files (the "Software"), to deal
 *	in the Software without restriction, including without limitation the rights
 *	to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *	copies of the Software, and to permit persons to whom the Software is
 *	furnished to do so, subject to the following conditions:
 *
 *	The above copyright notice and this permission notice shall be included in
 *	all copies or substantial portions of the Software.
 *
 *	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *	IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *	FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *	AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *	LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *	OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 *	THE SOFTWARE.
 */